% Estimation of the long-run risks model
clear all;
seed = 12345;
rng(seed, 'twister');
for dataf=1:2;

if dataf==1;
    % Quarterly data: 1952:Q2-2012:Q4
    load lrr_dataq.txt;  
    dg=100*lrr_dataq(2:end,3); % dividend growth
    cg=100*lrr_dataq(2:end,4); % consumption growth
elseif dataf==2;
    % Annual data: 1931-2009
    load lrr_datay.txt; 
    cg=100*lrr_datay(:,1);  % consumption growth
    dg=100*lrr_datay(:,3);  % dividend growth
end
    
options=optimset('Display','off','Algorithm','interior-point');
nsim=100; % number of simulated paths for SMD
lag=1;    % number of lags for SMD

% OLS and IV estimation
y=dg(2:end);
X=cg(2:end);
lagX=cg(1:end-1);
T=length(X);
reg=[ones(length(X),1) X];
k=size(reg,2);
%olsr=nwest(y,reg,4);
[olsr.beta,olsr.bint,V] = regress(y,reg); 
ols.sig=(V'*V)/(length(reg)-2);
ols.res=[olsr.beta; ols.sig];
reg1=[ones(length(X),1) lagX];
iv.beta=inv(reg1'*reg)*(reg1'*y); % IV estimation
iv.resid=y-reg*iv.beta;
iv.sig=(iv.resid'*iv.resid)/(length(reg)-2);
iv.res=[iv.beta; iv.sig];

% Data construction for CMD and SMD estimation
data.mu=mean(X);
demeanX=X-repmat(data.mu,T,1);
Gam0=[]; GamV=[]; GamVx=[];
for j=0:2;
    Gam0(j+1)=demeanX(j+1:end,:)'*demeanX(1:T-j)/T;
    GamV(j+1)=V(j+1:end,:)'*V(1:end-j)/T; 
    GamVx(j+1)=V(j+1:end,:)'*demeanX(1:end-j)/T;
end; 
data.Gam0=Gam0(1:lag+1)';
datamom=[olsr.beta; GamV(1); GamV(2); GamVx(2)];
data.mom=datamom;
data.mom0=datamom(1:end-1);
data.coef0=datamom(1:end-1)';
data.coef=datamom';   % empirical moments to be matched 
data.A=[Gam0(1) 2*Gam0(2); Gam0(2) Gam0(1)+Gam0(3)];
data.B=[Gam0(1); Gam0(2)];
data.X=X;
data.lagX=lagX;

% CMD estimation
init0=[0; 0.5; GamV(1); 0.5*Gam0(1)];
Ac=zeros(2,size(init0,1));
Ac(1,3)=-1;
Ac(2,4)=-1;
bc=[-0.0001; -0.0001];
[cmd0,~,~]=fmincon(@lrr_objcmd0,init0,Ac,bc,[],[],[],[],[],options,data);

% SMD estimation
err.eps=randn(T,nsim); % errors used in simulation estimation
err.uy=randn(T,nsim);
Ac=zeros(3,size(init0,1));
Ac(1,3)=-1;
Ac(2,4)=-1;
Ac(3,4)=1;
bc=[-0.0001; -0.0001; Gam0(1)-(2*(Gam0(2)^2))/(Gam0(1)+Gam0(3))];
[smd0,~,~]=fmincon(@lrr_objsmd0,init0,Ac,bc,[],[],[],[],[],options,data,err,nsim,lag);

est=NaN(4,4);
est(1:3,1)=ols.res;
est(1:3,2)=iv.res;
est(:,3)=cmd0;
est(:,4)=smd0;


if dataf==1;
    fprintf(' Estimates: quarterly data\n')
elseif dataf==2;
    fprintf(' Estimates: annual data\n')
end
fprintf('     OLS     IV     CMD     SMD\n')
fprintf('%8.3f%8.3f%8.3f%8.3f\n',est')
end